<?php
class Gateway_Ipasspay_Model_Payment extends Mage_Payment_Model_Method_Abstract {
	protected $_code = 'ipasspay_payment';
	protected $_formBlockType = 'ipasspay/form';
	protected $_isGateway = false;
	protected $_canAuthorize = true;
	protected $_canCapture = true;
	protected $_canCapturePartial = false;
	protected $_canRefund = false;
	protected $_canVoid = false;
	protected $_canUseInternal = false;
	protected $_canUseCheckout = true;
	protected $_canUseForMultishipping = false;
	public $_cardid;
	public $_ex_month;
	public $_ex_year;
	public $_cvv2;
	protected $_order = null;
	function getCheckout() {
		return Mage::getSingleton('checkout/session');
	}
	function getOrder() {
		if ($this->_order == null) {
			$session = $this->getCheckout();
			$this->_order = Mage::getModel('sales/order');
			$this->_order->loadByIncrementId($session->getLastRealOrderId());
		}
		return $this->_order;
	}
	public function capture(Varien_Object $payment, $amount) {
		$payment->setStatus(self::STATUS_APPROVED)->setLastTransId($this->getTransactionId());
		return $this;
	}
	public function getQuote() {
		return $this->getCheckout()->getQuote();
	}
	// 读获取支付网关地址 get gateway url
	public function getIpasspayUrl() {
		$form_url = $this->getConfigData('paymenturl');
		if ($form_url == "") {
			$form_url = "https://www.ipasspay.biz/index.php/Gateway/securepay";
		}
		return $form_url;
	}
	// 交易异步通知URL. get asyn_url
	protected function getNotifyURL() {
		return Mage::getUrl('ipasspay/payment/notify', array(
			'_secure' => true,
		));
	}
	// 交易即时通知的URL get syn_url
	function getBackUrl() {
		return Mage::getUrl('ipasspay/payment/return', array(
			'_secure' => true,
		));
	}
	function getOrderPlaceRedirectUrl() {
		$_cardid = isset($_POST["cardid"]) ? $_POST["cardid"] : "";
        $_ex_month = isset($_POST["ex_month"]) ? $_POST["ex_month"] : "";
		$_ex_year = isset($_POST["ex_year"]) ? $_POST["ex_year"] : "";
		$_cvv2 = isset($_POST["cvv2"]) ? $_POST["cvv2"] : "";

		$_SESSION['card_no'] = $_cardid;
		$_SESSION['card_ex_month'] = $_ex_month;
		$_SESSION['card_ex_year'] = $_ex_year;
		$_SESSION['card_cvv'] = $_cvv2;

		return Mage::getUrl('ipasspay/payment/redirect');
	}
	public function createFormBlock($name) {
		$block = $this->getLayout()->createBlock('ipasspay/form_payment', $name);
		$block->setMethod($this->_code);
		$block->setPayment($this->getPayment());
		return $block;
	}
	function getStandardCheckoutFormFields() {
		$_order = $this->getOrder();
		if (!($_order instanceof Mage_Sales_Model_Order)) {
			Mage::throwException($this->_getHelper()->__('Cannot retrieve order object'));
		}
		if (!empty($_order)) {
			$billing = $_order->getBillingAddress();
			$shipping = $_order->getShippingAddress();
		} else {
			Mage::throwException($this->_getHelper()->__('This order is empty!'));
		}
		$storeCurrency = Mage::getSingleton('directory/currency')->load($_order->getBaseCurrencyCode());
		$price_currency = Mage::app()->getStore()->getCurrentCurrencyCode(); // 订单标价币种
		$price_amount = sprintf('%.2f', $storeCurrency->convert($_order->getBaseGrandTotal(), $price_currency)); // 交易金额

		$Amount = $price_amount;
		$currency = $price_currency;
		$mid = $this->getConfigData('mid'); //商户id
		$api_key = $this->getConfigData('md5key');
		$site_id = $this->getConfigData('siteid'); //网站id
		$oid = $_order->getRealOrderId(); // 商户订单号

		$bill_firstname = $shipping->getFirstname();
		$bill_lastname = $shipping->getLastname();
		$bill_street = $billing->getStreet(1); // 账单地址
		$bill_city = $billing->getCity(); // 账单城市
		$bill_state = $billing->getRegionCode() ? $billing->getRegionCode() : "--"; // 账单地区
		$bill_country = $billing->getCountryModel()->getIso2Code(); // 账单国家, 转化为2位国家代码
		$bill_zip = $billing->getPostcode() ? $billing->getPostcode() : "--"; // 账单邮编
		$bill_phone = $billing->getTelephone() ? $billing->getTelephone() : "--"; // 账单电话
		$bill_email = $_order->getCustomerEmail(); // 账单email

		$syn_url = $this->getBackUrl(); // 交易即时通知的URL
		$asyn_url = $this->getNotifyURL(); // 交易异步通知的URL

		//$md5_info = md5($Amount . $currency . $mid . $site_id . $oid . $md5_key); //md5加密

		$hash_info = hash("sha256", $mid . $site_id .
			$oid . $Amount . $currency . $api_key);

		//$data_type = "xml"; // 返回数据格式 xml,json

		$basicinfo = array(
			"order_amount" => $Amount,
			"order_currency" => $currency,
			"mid" => $mid,
			"site_id" => $site_id,
			"oid" => $oid,
			"hash_info" => $hash_info,
			'bill_firstname' => $bill_firstname,
			'bill_lastname' => $bill_lastname,
			'bill_street' => $bill_street,
			'bill_city' => $bill_city,
			'bill_state' => $bill_state,
			'bill_country' => $bill_country,
			'bill_zip' => $bill_zip,
			'bill_phone' => $bill_phone,
			'bill_email' => $bill_email,
			'syn_url' => $syn_url,
			'asyn_url' => $asyn_url,
			'source_ip' => $this->getIPaddress(),
			'source_url' => $_SERVER["HTTP_REFERER"], //来源URL地址
			'gateway_version' => "1.0",
			"uuid" => $this->create_guid(),
		);

		$str = "";
		foreach ($basicinfo as $strkey => $value) {
			if (is_array($value)) {
				foreach ($value as $values => $strvalue) {
					$str .= $strkey . "=" . urlencode($strvalue) . "&";
				}
			} else {
				$str .= $strkey . "=" . urlencode($value) . "&";
			}
		}
		$_parameter = rtrim($str, '&');
		return $_parameter;
	}

	/**
	 * the function for creating uuid in PHP
	 */
	function create_guid() {
		$charid = strtoupper(md5(uniqid(mt_rand(), true)));
		$hyphen = chr(45); // "-"
		$uuid = substr($charid, 0, 8) . $hyphen
		. substr($charid, 8, 4) . $hyphen
		. substr($charid, 12, 4) . $hyphen
		. substr($charid, 16, 4) . $hyphen
		. substr($charid, 20, 12);
		return $uuid;
	}

	// 获取客户端的ip
	function getIPaddress() {
		$IPaddress = '';
		if (isset($_SERVER)) {
			if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
				$IPaddress = $_SERVER["HTTP_X_FORWARDED_FOR"];
			} else if (isset($_SERVER["HTTP_CLIENT_IP"])) {
				$IPaddress = $_SERVER["HTTP_CLIENT_IP"];
			} else {
				$IPaddress = $_SERVER["REMOTE_ADDR"];
			}
		} else {
			if (getenv("HTTP_X_FORWARDED_FOR")) {
				$IPaddress = getenv("HTTP_X_FORWARDED_FOR");
			} else if (getenv("HTTP_CLIENT_IP")) {
				$IPaddress = getenv("HTTP_CLIENT_IP");
			} else {
				$IPaddress = getenv("REMOTE_ADDR");
			}
		}
		$ips = explode(",", $IPaddress);
		return $ips[0];
	}

	function getIp() {
		$ipArr = array();

		$ipArr[] = 'HTTP_CLIENT_IP';
		$ipArr[] = 'HTTP_X_FORWARDED_FOR';
		$ipArr[] = 'REMOTE_ADDR';

		if (!empty($_SERVER["HTTP_CLIENT_IP"])) {
			$ipArr[0] = $_SERVER["HTTP_CLIENT_IP"];
		}
		if (!empty($_SERVER["HTTP_X_FORWARDED_FOR"])) {
			$proxy_ips = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
			$proxy_ips = array_reverse($proxy_ips);
			$proxy_ip = array_shift($proxy_ips);
			$ipArr[1] = $proxy_ip;
			$ipArr = array_merge($ipArr, $proxy_ips);
		}
		if (!empty($_SERVER["REMOTE_ADDR"])) {
			$ipArr[2] = $_SERVER["REMOTE_ADDR"];
		} else {
			$ipArr[] = "127.0.0.1";
		}

		foreach ($ipArr as $key => $ip) {
			$ip2long = bindec(decbin(ip2long($ip)));
			if ($ip2long != 0) {
				return $ip;
			}
		}
		return "127.0.0.1";

	}
}
